
#define tolX 1e-8 // tolerance of largest element of search direction
#define tolFun 1e-8 // tolerance of directional derivative
#define maxIter 500 // maximum number of iteration allowed
#define constant1 1e8	// constant1 = 1e8
#define constant2 1e-8	// constant2 = 1e-8
#include "nrutil.h"

Bool nlsqLM(int, double*, int, double*, int);

/* extern double **matrix(long nrl, long nrh, long ncl, long nch); */
/* extern int *ivector(long nl, long nh); */
/* extern double *dvector(long nl, long nh); */
/* extern double **dmatrix(long nrl, long nrh, long ncl, long nch); */
/* extern void free_vector(double *v, long nl, long nh); */
/* extern void free_matrix(double **m, long nrl, long nrh, long ncl, long nch); */

	// sub-functions
	void searchq(int *PCNTp, double *fnewp, double *MATL, double *MATX, double gdold, double *stepsizep);
	double cubici1(double f2, double f1, double c2, double c1, double dx);
	double cubici2(double c, double f1, double f2, double f3, double x1, double x2, double x3);
	void cubici3(double f2, double f1, double c2, double c1, double dx, double *sp, double *fp);
	void vecassign(double *vec1, double *vec2, int veclength);
	void matassign(double **mat1, double **mat2, int rows, int cols);
	void vecproduct(double *vec1, double *vec2, double *result, int veclength);
	void vecmatproduct(double **mat, double *vec1, double *vec2, int m, int n);
	void matproduct( double **mat1, double **mat2, double **mat3, int l, int m, int n );
	void maxv(double *a, int length, double *maxa);
	void minv(double *a, int length, double *mina);
	void transpose(double **mat1, double **mat2, int rows, int cols);
	Bool svdcmp(double **a, int m, int n, double *w, double **v);
	double pythag(double a, double b);

Bool nlsqLM(int mIndex, double *x, int nvar, double *CostFunction, int nfun )
{
	// NLSQLM Solves non-linear least squares problems with LevenbergMarquardt method
	// The problems should be of the form:
	// min  sum {FUN(X).^2}    where FUN and X may be vectors or matrices.
	// x


	// Definition of variables
	int gcnt, i, intcaller, iter, j, maxFunEvals, *nonZeroSum;
	int numFunEvals, numGradEvals, PCNT, status;
	double *CHG, CHGtemp1, CHGtemp2, condJAC, currstepsize, *estf;
	double EstSum, fbest, FIRSTF, fval, gdnew, GDOLD, *GradF, GradFactor, **Hessian;
	double *HessianS, **HessianV, **JAC, **JACFD;
	double *JACS, **JACV, *MATL, *MATX, maxS, minS, *newf, NewF;
	double newstep, *OLDF, **OLDJ, *OLDX, *OX, *SD;
	double steplen, stepsize, *sumabsJAC, temp, **tempHessian, **tempJAC;
	double *terminatepar, **transposeHessian;
	double **transposeJAC, *XOUT;
	Bool Result = True;
	
//	if (!calcError(mIndex))         moved into DoOptimization
//		return False;

	// Initialization of vector and matrix of variables
	nonZeroSum = ivector(1,nvar);
	CHG = dvector(1,nvar);
	estf = dvector(1,nfun);
	GradF = dvector(1,nvar);
	Hessian = dmatrix(1,nvar,1,nvar);
	HessianS = dvector(1,nvar);
	HessianV = dmatrix(1,nvar,1,nvar);
	JAC = dmatrix(1,nfun,1,nvar);
	JACFD = dmatrix(1,nfun,1,nvar);
	JACS = dvector(1,nvar);
	JACV = dmatrix(1,nvar,1,nvar);
	MATL = dvector(1,3);
	MATX = dvector(1,3);
	newf = dvector(1,nfun);
	OLDF = dvector(1,nfun);
	OLDJ = dmatrix(1,nfun,1,nvar);
	OLDX = dvector(1,nvar);
	OX = dvector(1,nvar);
	SD = dvector(1,nvar);
	sumabsJAC = dvector(1,nvar);
	tempHessian = dmatrix(1,nvar,1,nvar);
	tempJAC = dmatrix(1,nfun,1,nvar);
	terminatepar = dvector(1,3);
	transposeHessian = dmatrix(1,nvar,1,nvar);
	transposeJAC = dmatrix(1,nvar,1,nfun);
	XOUT = dvector(1,nvar);

	//Initialization of scalar variables
	intcaller=1;	// intcaller = 0 for cubic interpolation; 1 for quadratic cubic mixed interpolation
	gcnt=i=iter=j=numFunEvals=numGradEvals=PCNT=0;
	CHGtemp1=CHGtemp2=condJAC=fbest=FIRSTF=fval=gdnew=GDOLD=maxS=minS=newstep=NewF=steplen=stepsize=temp=0.0;
	status=-1; // status=-1: SD initialization; status=1: obtaining minimizer and program termination; status=0: o/w
	currstepsize=1.0; // current stepsize
	EstSum = 0.5; // estimated square error
	maxFunEvals=100*nvar; // maximum number of function evaluations allowed
	vecassign(x, XOUT, nvar);
	vecproduct(CostFunction, CostFunction, &temp, nfun);
	MATL[1] = temp, MATL[2] = 0.0, MATL[3] = 0.0;
	FIRSTF = temp;
	vecassign(XOUT, OLDX, nvar);
	vecassign(CostFunction, OLDF, nfun);
	//sprintf(['\n                                                     Directional \n',...
	//           ' Iteration  Func-count    Residual     Step-size      derivative    Lambda']));

	// overdetermined system of equations, determine lamda(GradFactor)
	if (nfun >= nvar) GradFactor = 0;
	else GradFactor = 1;
	for (i=1; i<=nvar; i++) {
		if (XOUT[i] >= 0) CHG[i]=0.0000001*(1+XOUT[i]);
		else CHG[i]=0.0000001*(1-XOUT[i]);
	}

	// main loop
	while (status != 1) {
		iter++;

		// Work Out Gradients
		vecassign(CostFunction, OLDF, nfun);
		for ( i=1; i<=nvar; i++ ) {		//get CHG
			if (CHG[i] + eps >=0)	CHGtemp1=1;
			else	CHGtemp1=-1;
			if (CHG[i] >= 0.1)	CHGtemp2=0.1;
			else if (CHG[i] >= 0.00000001)	CHGtemp2=CHG[i];
			else if (CHG[i] >= -0.00000001)	CHGtemp2=0.00000001;
			else if (CHG[i] >= -0.1) CHGtemp2= -CHG[i];
			else	CHGtemp2=0.1;
			CHG[i]=CHGtemp1*CHGtemp2;
		}
		for (gcnt=1; gcnt<=nvar; gcnt++) {	// get Jacobi
			temp = XOUT[gcnt];
			XOUT[gcnt] = temp +CHG[gcnt];
			vecassign(XOUT, x, nvar);

			//CostFunction = feval(fun,x,varargin);
			//calculate(x, nvar, CostFunction, nfun);
			if (!calcError(mIndex)){
				Result = False;
				goto Finish;
			}
	
			for (i=1; i<=nfun; i++) {
				JACFD[i][gcnt]=(CostFunction[i] - OLDF[i])/CHG[gcnt];
			}
			XOUT[gcnt] = temp;
		}
		vecassign(OLDF, CostFunction, nfun);
		numFunEvals = numFunEvals + nvar;
		matassign(JACFD, JAC, nfun, nvar);	// let JAC = JACFD

		// Try to set difference to 1e-8 for next iteration
		// JAC is of nfunXnvar
		// nfun==1 then sumabsJAC=abs(JAC): row vector of length nvar
		// nfun!=1 then sumabsJAC=sum(abs(JAC)); : column vector of length nvar
		for (i=1; i<=nvar; i++) {
			sumabsJAC[i] = 0.0;
			for (j=1;j<=nfun;j++) {
				sumabsJAC[i] += fabs(JAC[j][i]);
			}
		}
// *******************************************| CHECK Inf AND eps VALUES
		for (i=1;i<=nvar;i++) {
			if (sumabsJAC[i] == 0)	CHG[i] = Inf;	// to avoid divided by zero error
			else CHG[i] = nfun*constant2/sumabsJAC[i];
		}
		transpose(JAC, transposeJAC, nfun, nvar);
		vecmatproduct(transposeJAC, CostFunction, GradF, nvar, nfun);	// GradF = 2*(CostFunction'*JAC)';
		for (i=1; i<=nvar; i++) {
			GradF[i] = 2*GradF[i];
		}
		vecproduct(CostFunction, CostFunction, &NewF, nfun);
		
		// ---------------Initialization of Search Direction------------------
		if (status==-1) {
			matassign(JAC, tempJAC, nfun, nvar); // save JAC, since svdcmp will modify the original matrix
			if (!svdcmp(JAC, nfun, nvar, JACS, JACV)){Result=False; goto Finish;}  // SVD of JAC
			matassign(tempJAC, JAC, nfun, nvar); // restore JAC
			maxv(JACS, nvar, &maxS); // largest and smallest singlar value of JAC
			minv(JACS, nvar, &minS);
			if (minS<eps) condJAC = Inf;
			else condJAC = maxS/minS;

			// obtain search direction
			if (condJAC>constant1) {   // bad matrix
				GradFactor = maxS+1; // GradFactor=norm(JAC)+1;
				transpose(JAC, transposeJAC, nfun, nvar);
				//SD=-(JAC'*JAC+(norm(JAC)+1)*(eye(nvar,nvar)))\(CostFunction'*JAC)';
				matproduct(transposeJAC, JAC, Hessian, nvar, nfun, nvar);
				for (i=1; i<=nvar; i++) Hessian[i][i] += maxS+1; // update Hessian
				matassign(Hessian, tempHessian, nvar, nvar); // save Hessian
				if (!svdcmp(Hessian, nvar, nvar, HessianS, HessianV)){Result=False; goto Finish;} // SVD of Hessian
				vecmatproduct(transposeJAC, CostFunction, SD, nvar, nfun); // the following block is for computing SD: SD = -V*inv(S)*U'*JAC'*CostFunction
				transpose(Hessian, transposeHessian, nvar, nvar);
				vecmatproduct(transposeHessian, SD, SD, nvar, nvar);
				for (i=1; i<=nvar; i++) {
					if (HessianS[i]<eps) SD[i] = -Inf;
					else SD[i] /= -HessianS[i];
				}
				vecmatproduct(HessianV, SD, SD, nvar, nvar);
			}
			else {	// good matrix
				transpose(JAC, transposeJAC, nfun, nvar);
				// SD=-(JAC'*JAC+GradFactor*(eye(nvar,nvar)))\(CostFunction'*JAC)';
				matproduct(transposeJAC, JAC, Hessian, nvar, nfun, nvar);
				for (i=1; i<=nvar; i++) {
					Hessian[i][i] += GradFactor; // update Hessian
				}
				matassign(Hessian, tempHessian, nvar, nvar); // save Hessian
				if (!svdcmp(Hessian, nvar, nvar, HessianS, HessianV)) {Result=False; goto Finish;} // SVD of Hessian
				vecmatproduct(transposeJAC, CostFunction, SD, nvar, nfun); // the following block is for computing SD: SD = -V*inv(S)*U'*JAC'*CostFunction
				transpose(Hessian, transposeHessian, nvar, nvar);
				vecmatproduct(transposeHessian, SD, SD, nvar, nvar);
				for (i=1; i<=nvar; i++) {
					if (HessianS[i]<eps) SD[i] = -Inf;
					else SD[i] = -SD[i]/HessianS[i];
				}
				vecmatproduct(HessianV, SD, SD, nvar, nvar);
			}

			FIRSTF = NewF;
			matassign(JAC, OLDJ, nfun, nvar); // OLDJ = JAC
			vecproduct(GradF, SD, &GDOLD, nvar); // get GDOLD: directional derivative of F
			vecmatproduct(JAC, SD, newf, nfun, nvar);	// newf = JAC*SD + CostFunction
			for (i=1; i<=nfun; i++) {
				newf[i] += CostFunction[i];
			}
			vecproduct(newf, newf, &GradFactor, nfun); // GradFactor = newf'*newf

			//SD=-(JAC'*JAC+GradFactor*(eye(nvar,nvar)))\(CostFunction'*JAC)';
			matproduct(transposeJAC, JAC, Hessian, nvar, nfun, nvar);
			for (i=1; i<=nvar; i++) {
				Hessian[i][i] += GradFactor; // update Hessian
			}
			matassign(Hessian, tempHessian, nvar, nvar); // save Hessian
			if (!svdcmp(Hessian, nvar, nvar, HessianS, HessianV)) {Result=False; goto Finish;} // SVD of Hessian
			vecmatproduct(transposeJAC, CostFunction, SD, nvar, nfun); // the following block is for computing SD: SD = -V*inv(S)*U'*JAC'*CostFunction
			transpose(Hessian, transposeHessian, nvar, nvar);
			vecmatproduct(transposeHessian, SD, SD, nvar, nvar);
			for (i=1; i<=nvar; i++) {
				if (HessianS[i]<eps) SD[i] = -Inf;
				else SD[i] /= -HessianS[i];
			}
			vecmatproduct(HessianV, SD, SD, nvar, nvar);

			vecmatproduct(JAC, SD, newf, nfun, nvar);	//newf=JAC*SD+CostFunction;
			for (i=1; i<=nfun; i++) {
				newf[i] += CostFunction[i];
			}
			for (i=1; i<=nvar; i++) {
				XOUT[i] += currstepsize*SD[i];
			}
			vecproduct(newf, newf, &EstSum, nfun);
			status = 0;
			PCNT=intcaller;
		}

		//-------------Direction Update------------------
		else {
			vecproduct(GradF, SD, &gdnew, nvar);
			//num=sprintf(formatstr,iter,numFunEvals,NewF,currstepsize,gdnew);
			if (gdnew>0 && NewF>FIRSTF) {
				// Case 1: New function is bigger than last and gradient w.r.t. SD -ve
				// ... interpolate.
			  stepsize=cubici1(NewF,FIRSTF,gdnew,GDOLD,currstepsize);
			  //	currstepsize=0.9*stepsize;
			  currstepsize = 10*stepsize;
			}
			else if (NewF<FIRSTF) {
				// New function less than old fun. and OK for updating
				//         .... update and calculate new direction.
				cubici3(NewF, FIRSTF, gdnew, GDOLD, currstepsize, &newstep, &fbest);
				if (fbest>NewF) fbest=0.9*NewF;
				if (gdnew<0) {
					if (newstep<currstepsize) newstep = 2*currstepsize+0.0001;
					currstepsize = fabs(newstep);
				}
				else {
					if (currstepsize>0.9) {
						if (fabs(newstep)>1) currstepsize=1;
						else currstepsize = fabs(newstep);
					}
				}
				currstepsize=fabs(currstepsize);
				// Levenberg_marquardt Method N.B. EstSum is the estimated sum of squares.
				//                                 GradFactor is the value of lambda.
				// Estimated Residual:
				if (EstSum>fbest) GradFactor=GradFactor/(1+currstepsize);
				else GradFactor=GradFactor+(fbest-EstSum)/(currstepsize+eps);
				// SD=-(JAC'*JAC+GradFactor*(eye(nvar,nvar)))\(CostFunction'*JAC)';
				matproduct(transposeJAC, JAC, Hessian, nvar, nfun, nvar);
				for (i=1; i<=nvar; i++) {
					Hessian[i][i] += GradFactor; // update Hessian
				}
				matassign(Hessian, tempHessian, nvar, nvar); // save Hessian
				if (!svdcmp(Hessian, nvar, nvar, HessianS, HessianV)) {Result=False; goto Finish;} // SVD of Hessian
				vecmatproduct(transposeJAC, CostFunction, SD, nvar, nfun); // the following block is for computing SD: SD = -V*inv(S)*U'*JAC'*CostFunction
				transpose(Hessian, transposeHessian, nvar, nvar);
				vecmatproduct(transposeHessian, SD, SD, nvar, nvar);
				for (i=1; i<=nvar; i++) {
					if (HessianS[i]<eps) SD[i] = -Inf;
					else SD[i] /= -HessianS[i];
				}
				vecmatproduct(HessianV, SD, SD, nvar, nvar);

				currstepsize=1;
				vecmatproduct(JAC, SD, estf, nfun, nvar);	// estf = JAC*SD + CostFunction
				for (i=1; i<=nfun; i++) {
					estf[i] += CostFunction[i];
				}
				vecproduct(estf, estf, &EstSum, nfun);	// EstSum = estf'*estf
				
				vecproduct(GradF, SD, &gdnew, nvar); // gdnew=GradF'*SD;
				vecassign(XOUT, OLDX, nvar);	// OLDX = XOUT

				// Save Variables
				FIRSTF = NewF;
				GDOLD = gdnew;
				PCNT=intcaller; // If quadratic interpolation set PCNT
				if (intcaller==1) {
					MATX[1] = 0.0, MATX[2] = 0.0, MATX[3] = 0.0;
					MATL[1] = NewF;
				}
			}
			else { // Halve Step-length
				if (NewF==FIRSTF) {
				  //printf("No improvement in search direction: Terminating.\n");
					status = 1;
				}
				else {
					currstepsize=currstepsize/8;
					if (currstepsize<constant2) currstepsize = -currstepsize;
				}
			}
			for (i=1; i<=nvar; i++) {
				XOUT[i] = OLDX[i] + currstepsize*SD[i];
			}
			//disp([num,'       ',how])
		}	//----------End of Direction Update-------------------

		PCNT=intcaller;
		if (intcaller==1) {
			MATX[1] = 0.0, MATX[2] = 0.0, MATX[3] = 0.0;
			MATL[1] = NewF;
		}

		//--------------Check Termination-----------------
		// terminatepar[1] = max(abs(SD)), terminatepar[2] = GradF'*SD, terminatepar[3] = max(abs(GradF))
		terminatepar[1] = fabs(SD[1]), terminatepar[2] = 0, terminatepar[3] = fabs(GradF[1]);
		for (i=1; i<=nvar; i++) {	// assign vector terminatepar
			if (fabs(SD[i]) > terminatepar[1]) terminatepar[1] = fabs(SD[i]);
			terminatepar[2] += fabs(GradF[i]*SD[i]); 
			if (fabs(GradF[i])>terminatepar[3]) terminatepar[3] = fabs(GradF[i]);
		}
		if (terminatepar[1]<tolX) {	// termination case 1
		  //printf("Optimization terminated successfully: Search direction less than tolX. \n");
			status = 1;
		}
		else if (terminatepar[2]<tolFun && terminatepar[3]<10*(tolFun+tolX)) { // termination case 2
		  //printf("Optimization terminated successfully: Gradient in the search direction less than tolFun\n");
		  //printf("                                      Gradient less than 10*(tolFun+tolX).\n");
			status = 1;
		}
		else if (numFunEvals>maxFunEvals) { // termination case 3
		  //printf("Maximum number of function evaluations exceeded.\n");
			status = 1;
		}
		else if (iter>maxIter) { // termination case 4
		  //printf("Maximum number of iterations exceeded.\n");
			status = 1;
		}
		else { // termination case 5, none of the criteria above is satisfied, do not terminate
			// Line search using mixed polynomial interpolation and extrapolation.
			if ( PCNT != 0 ) {
				vecassign(XOUT, OX, nvar);
				vecassign(CostFunction, OLDF, nfun);
				while ( PCNT > 0 && numFunEvals <= maxFunEvals ) {
					vecassign(XOUT, x, nvar);

					//CostFunction = feval(fun,x,varargin);
					//	calculate(x, nvar, CostFunction, nfun);
					if (!calcError(mIndex))
						{Result=False; goto Finish;}

					numFunEvals++;
					vecproduct(CostFunction, CostFunction, &NewF, nfun);
					// <= used in case when no improvement found.
					temp = 0.0;
					vecproduct(OLDF, OLDF, &temp, nfun);
					if ( NewF<=temp ) {
						vecassign(XOUT, OX, nvar);
						vecassign(CostFunction, OLDF, nfun);
					}

					//[PCNT,MATL,MATX,steplen,NewF,how]=searchq(PCNT,NewF,MATL,MATX,GDOLD,currstepsize,how);
					steplen = currstepsize;
					searchq(&PCNT,&NewF,MATL,MATX,GDOLD,&steplen); // mixed interpolation
					currstepsize=steplen;
					for (i=1; i<=nvar; i++) XOUT[i] = OLDX[i] + steplen*SD[i];
					if ( NewF==FIRSTF ) PCNT=0;
				} // end while
				vecassign(OX, XOUT, nvar);
				vecassign(OLDF, CostFunction, nfun);
				if ( numFunEvals>maxFunEvals ) {
				  //printf("Maximum number of function evaluations exceeded.\n");
					status=1;
				}
			} // if PCNT!=0
		} // if (terminatepar[1] < tolX)
		vecassign(XOUT, x, nvar);

		//CostFunction = feval(fun,x, varargin);
		//	calculate(x, nvar, CostFunction, nfun);
		if (!calcError(mIndex)){
			Result = False;
			goto Finish;
		}

		// JAC will be updated when it is finite-differenced
		numFunEvals = numFunEvals + 1;

	} // while

	vecassign(OLDX, XOUT, nvar);
	vecassign(XOUT, x, nvar);

Finish:
	free_ivector(nonZeroSum,1,nvar);
	free_vector(CHG,1,nvar);
	free_vector(estf,1,nfun);
	free_vector(GradF,1,nvar);
	free_matrix(Hessian,1,nvar,1,nvar);
	free_vector(HessianS,1,nvar);
	free_matrix(HessianV,1,nvar,1,nvar);
	free_matrix(JAC,1,nfun,1,nvar);
	free_matrix(JACFD,1,nfun,1,nvar);
	free_vector(JACS,1,nvar);
	free_matrix(JACV,1,nvar,1,nvar);
	free_vector(MATL,1,3);
	free_vector(MATX,1,3);
	free_vector(newf,1,nfun);
	free_vector(OLDF,1,nfun);
	free_matrix(OLDJ,1,nfun,1,nvar);
	free_vector(OLDX,1,nvar);
	free_vector(OX,1,nvar);
	free_vector(SD,1,nvar);
	free_vector(sumabsJAC,1,nvar);
	free_matrix(tempHessian,1,nvar,1,nvar);
	free_matrix(tempJAC,1,nfun,1,nvar);
	free_vector(terminatepar,1,3);
	free_matrix(transposeHessian,1,nvar,1,nvar);
	free_matrix(transposeJAC,1,nvar,1,nfun);
	free_vector(XOUT,1,nvar);

	return Result;
	// currently the function can return
	// XOUT: the solution of the minimization
	// fval: the squared error;
	// iter: the total iterations the program takes;
	// numFunEvals: the number of total function evaluations
	/*printf("CostFunction is:\n");
	for (i=1;i<=nfun;i++) {
		printf("%f,\n", CostFunction[i]);
	}
	printf("\n");

	printf("XOUT is:\n");
	for (i=1;i<=nvar;i++) {
		printf("%f,\n", XOUT[i]);
	}
	printf("\n");


	vecproduct(CostFunction, CostFunction, &fval, nfun);
	printf("the mean square error is: %f\n", fval);*/
}


/************************************** searchq: mixed quadratic and cubic polynomial interpolation ****************************/

void searchq(int *PCNTp, double *fnewp, double *MATL, double *MATX, double gdold, double *stepsizep)
// [pcnt, MATL,MATX,stepsize,fnew,how]=searchq(pcnt,fnew,MATL,MATX,gdold,stepsize,how)
{
	double newstep, temp1, temp2;
	int PCNT=*PCNTp;
	double fnew=*fnewp, stepsize=*stepsizep;

    if (PCNT==1) {
        // Case 1: Next point less than initial point.
		//     Increase step-length based on last gradient evaluation
		if (fnew<MATL[1]) {
            // Quadratic Extrapolation using gradient of first point and
            // values of two other points.
            MATL[2]=fnew;
            MATX[2]=stepsize;
            newstep=-0.5*gdold*stepsize*stepsize/(fnew-gdold*stepsize-MATL[1]+eps);
            if (newstep<stepsize) {
                newstep=1.2*stepsize;
            }
            stepsize=1.2*newstep;
            PCNT=2;
		}
		else {
            // Case 2: New point greater than initial point. Decrease step-length.
            MATL[3]=fnew;
            MATX[3]=stepsize;
            //Interpolate to get stepsize
			// stepsize=max([constant2*stepsize,-gdold*0.5*stepsize^2/(fnew-gdold*stepsize-MATL[1]+eps)]);
			temp1 = constant2*stepsize;
			temp2 = -gdold*0.5*stepsize*stepsize/(fnew-gdold*stepsize-MATL[1]+eps);
			if (temp1>temp2) stepsize=temp1;
			else stepsize=temp2;
            PCNT=3;
		}
    }
    // Case 3: Last run was Case 1 (pcnt=2) and new point less than
    //     both of other 2. Replace.
    else if (PCNT==2  && fnew<MATL[2]) {
        newstep=cubici2(gdold, MATL[1], MATL[2], fnew, MATX[1], MATX[2], stepsize);
		MATL[1]=MATL[2];
		MATX[1]=MATX[2];
		MATL[2]=fnew;
		MATX[2]=stepsize;
		// stepsize=min([newstep,1])+1.5*stepsize;
		if (newstep<1) stepsize=newstep+1.5*stepsize;
		else stepsize=1+1.5*stepsize;
		if (newstep>stepsize) stepsize=1.2*newstep;
		else stepsize=1.2*stepsize;
	}
    // Case 4: Last run was Case 2: (pcnt=3) and new function still
    //     greater than initial value.
    else if (PCNT==3 && fnew>=MATL[1]) {
        MATL[2]=fnew;
		MATX[2]=stepsize;
		if (stepsize<0.000001) {
            newstep=-stepsize/2;
            // Give up if the step-size gets too small
            // Stops infinite loops if no improvement is possible.
            if (fabs(newstep)<(eps*eps)) PCNT = 0;
		}
		else {
            newstep=cubici2(gdold, MATL[1], MATL[3], fnew, MATX[1], MATX[3], stepsize);
        }
		MATX[3]=stepsize;
		// if (isnan(newstep)) stepsize=stepsize/2;
        // else
		stepsize=newstep;
		MATL[3]=fnew;
    }
    // Otherwise must have Bracketed Minimum so do quadratic interpolation.
    //  ... having just increased step.
    else if (PCNT==2 && fnew>=MATL[2]) {
        MATX[3]=stepsize;
		MATL[3]=fnew;
		stepsize=cubici2(gdold,MATL[1], MATL[2], MATL[3], MATX[1], MATX[2], MATX[3]);
		PCNT=4;
    }
    // ...  having just reduced step.
    else if  (PCNT==3  && fnew<MATL[1]) {
        MATX[2]=stepsize;
        MATL[2]=fnew;
		stepsize=cubici2(gdold,MATL[1], MATL[2], MATL[3], MATX[1], MATX[2], MATX[3]);
		PCNT=4;
    }
    // Have just interpolated - Check to see whether function is any better
    // - if not replace.
    else if (PCNT==4) {
        PCNT=0;
        stepsize=fabs(stepsize);
		// If interpolation failed use old point.
		if (fnew>MATL[2]) {
            fnew=MATL[2];
            stepsize=MATX[2];
		}
    } // if pcnt==1

	*PCNTp=PCNT;
	*fnewp=fnew, *stepsizep=stepsize;
}


/************************************* cubic interpolation 1 *************************************/

double cubici1(double f2, double f1, double c2, double c1, double dx)
{
	double z, w, r, temp, tempf2=f2;

	if ( f2>Inf ) {
		f2 = 1/eps;
	}
	z = 3*(f1-f2)/dx+c1+c2;
	temp = z*z-c1*c2;
	if (temp>=0) w = sqrt(temp);
	else w = 0;
	r = dx*((z+w-c1)/(c2-c1+2*w));
	f2=tempf2;
	return r;
}


/************************************* cubic interpolation 2 *************************************/

double cubici2(double c, double f1, double f2, double f3, double x1, double x2, double x3)
{
	double g1,g2,g3,temp1,temp2,temp3,h,a,b,step;

	g1 = f1 - c*x1;
	g2 = f2 - c*x2;
	g3 = f3 - c*x3;
	temp1 = g2-g3;
	temp2 = g3-g1;
	temp3 = g1-g2;
	g1=temp1; g2=temp2; g3=temp3;

	h=(x1-x2)*(x2-x3)*(x3-x1)*(x1*x2+x2*x3+x3*x1);
	a = 3*(x1*x1*g1+x2*x2*g2+x3*x3*g3)/h;
	b = (x1*x1*x1*g1+x2*x2*x2*g2+x3*x3*x3*g3)/h;

	// Find minimizer.
	if (b*b-a*c >= 0) {
		step = (b + sqrt(b*b-a*c))/a;
	}
	else {
		step = b/a;
	}

	// Is step acceptable?
	if (step < 0 || step>Inf) {
		g1=f2-f3; g2=f3-f1; g3=f1-f2;
		step = (x1*x1*g1+x2*x2*g2+x3*x3*g3)/(x1*g1+x2*g2+x3*g3)/2;
	}
	return step;
}


/************************************* cubic interpolation 3 *************************************/

void cubici3(double f2, double f1, double c2, double c1, double dx, double *sp, double *fp)
{
	double a,b,disc,tempf2=f2;
	double s=*sp, f=*fp;

	if (f2>Inf) f2 = 1/eps;
	a = 3*(c2*dx-2*f2+c1*dx+2*f1)/dx/dx/dx;
	b = -(c2*dx-3*f2+2*c1*dx+3*f1)/dx/dx;
	disc = b*b - a*c1;
	if (a == 0)	s = -c1/b;
	else if (disc <= 0)  s = -b/a;
	else  s = (-b+sqrt(disc))/a;

	if (s<0)  s = -s;
	f = a/3*s*s*s + b*s*s + c1*s + f1;
	f2=tempf2;

	*sp=s; *fp=f;
}

/************************************* vector assignment *************************************/

// assign a vector vec1 to another vector vec2, x and y should have the same length
void vecassign(double *vec1, double *vec2, int veclength)
{
	int i=0;
    for (i=1; i<=veclength; i++) {
		vec2[i] = vec1[i];
	}
}

/************************************* matrix assignment *************************************/

// assign a matrix mat1 to another matrix mat2, mat1 and mat2 should have the same size (rows, cols)
void matassign(double **mat1, double **mat2, int rows, int cols)
{
	int i,j;
    for (i=1; i<=rows; i++) {
		for (j=1; j<=cols; j++) {
			mat2[i][j] = mat1[i][j];
		}
    }
}

/************************************* vector multiplication *************************************/

// result=vec1'*vec2
void vecproduct(double *vec1, double *vec2, double *result, int veclength)
{
	int i=0;
	*result=0;
	for (i=1; i<=veclength; i++) {
		*result += vec1[i]*vec2[i];
	}
}


/************************************* matrix and vector multiplication *************************************/

// mat: mxn, vec2=mat*vec1
void vecmatproduct(double **mat, double *vec1, double *vec2, int m, int n)
{
	int i=0,j=0;
	double *tempvec;
	tempvec=dvector(1,n);
	for (i=1; i<=n; i++) {
		tempvec[i] = vec1[i];
	}

	for ( i=1; i<=m; i++ ) {
		vec2[i] = 0.0;
		for (j=1; j<=n; j++) {
			vec2[i] += mat[i][j]*tempvec[j];
		}
	}
	free_vector(tempvec,1,n);
}

/************************************* matrix multiplication *************************************/

// mat3 = mat1*mat2
// mat1: lxm, mat2: mxn, mat3: lxn
void matproduct( double **mat1, double **mat2, double **mat3, int l, int m, int n )
{
	int i=0,j=0,k=0;
	double **tempmat1, **tempmat2;
	tempmat1=dmatrix(1,l,1,m), tempmat2=dmatrix(1,m,1,n);
	for (i=1; i<=l; i++) {
		for (j=1; j<=m; j++) {
			tempmat1[i][j]=mat1[i][j];
		}
	}
	for (i=1; i<=m; i++) {
		for (j=1; j<=n; j++) {
			tempmat2[i][j]=mat2[i][j];
		}
	}

	for (i=1; i<=l; i++) {
		for (j=1; j<=n; j++) {
			mat3[i][j]=0;
			for (k=1; k<=m; k++) {
				mat3[i][j] += tempmat1[i][k]*tempmat2[k][j];
			}
		}
	}

	free_matrix(tempmat1,1,l,1,m);
	free_matrix(tempmat2,1,m,1,n);
}

/************************************* max of a vector *************************************/

void maxv(double *a, int length, double *maxa)
{
	int i=0;
	*maxa=a[1];
	for (i=1; i<=length; i++) {
		if (a[i]>(*maxa)) *maxa = a[i];
	}
}

/************************************* min of a vector *************************************/

void minv(double *a, int length, double *mina)
{
	int i=0;
	*mina=a[1];
	for (i=1; i<=length; i++) {
		if (a[i]<(*mina)) *mina = a[i];
	}
}

/************************************* transpose of a matrix *************************************/

// mat2 = mat1', the dimension should be correct, i.e. mat1 with (rows, cols) and mat2 with (cols, rows)
void transpose(double **mat1, double **mat2, int rows, int cols)
{
	int i=0,j=0;
	for (i=1; i<=rows; i++) {
		for (j=1; j<=cols; j++) {
			mat2[j][i] = mat1[i][j];
		}
	}
}

/******************************** SVD computation **************************************/

Bool svdcmp(double **a, int m, int n, double *w, double **v)

/* Given a matrix a[1..m][1..n], this routine computes its singular value decomposition, A =
UWV T. Thematrix U replaces a on output. The diagonal matrix of singular values W is output
as a vector w[1..n]. Thematrix V (not the transpose V T ) is output as v[1..n][1..n]. */

{
	double pythag(double a, double b);
	int flag,i,its,j,jj,k,l,nm;
	double anorm,c,f,g,h,s,scale,x,y,z,*rv1;
	rv1=dvector(1,n);
	g=scale=anorm=0.0;  // Householder reduction to bidiagonal form.
	for (i=1;i<=n;i++) {
		l=i+1;
		rv1[i]=scale*g;
		g=s=scale=0.0;
		if (i <= m) {
			for (k=i;k<=m;k++) scale += fabs(a[k][i]);
			if (scale) {
				for (k=i;k<=m;k++) {
					a[k][i] /= scale;
					s += a[k][i]*a[k][i];
				}
				f=a[i][i];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][i]=f-g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=i;k<=m;k++) s += a[k][i]*a[k][j];
					f=s/h;
					for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
				}
				for (k=i;k<=m;k++) a[k][i] *= scale;
			}
		}
		w[i]=scale *g;
		g=s=scale=0.0;
		if (i <= m && i != n) {
			for (k=l;k<=n;k++) scale += fabs(a[i][k]);
			if (scale) {
				for (k=l;k<=n;k++) {
					a[i][k] /= scale;
					s += a[i][k]*a[i][k];
				}
				f=a[i][l];
				g = -SIGN(sqrt(s),f);
				h=f*g-s;
				a[i][l]=f-g;
				for (k=l;k<=n;k++) rv1[k]=a[i][k]/h;
				for (j=l;j<=m;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[j][k]*a[i][k];
					for (k=l;k<=n;k++) a[j][k] += s*rv1[k];
				}
				for (k=l;k<=n;k++) a[i][k] *= scale;
			}
		}
		anorm=FMAX(anorm,(fabs(w[i])+fabs(rv1[i])));
	}
	for (i=n;i>=1;i--) {	// Accumulation of right-hand transformations.
		if (i < n) {
			if (g) {
				for (j=l;j<=n;j++)	// Double division to avoid possible underflow.
					v[j][i]=(a[i][j]/a[i][l])/g;
				for (j=l;j<=n;j++) {
					for (s=0.0,k=l;k<=n;k++) s += a[i][k]*v[k][j];
					for (k=l;k<=n;k++) v[k][j] += s*v[k][i];
				}
			}
			for (j=l;j<=n;j++) v[i][j]=v[j][i]=0.0;
		}
		v[i][i]=1.0;
		g=rv1[i];
		l=i;
	}
	for (i=IMIN(m,n);i>=1;i--) {	// Accumulation of left-hand transformations.
		l=i+1;
	g=w[i];
	for (j=l;j<=n;j++) a[i][j]=0.0;
	if (g) {
		g=1.0/g;
		for (j=l;j<=n;j++) {
			for (s=0.0,k=l;k<=m;k++) s += a[k][i]*a[k][j];
			f=(s/a[i][i])*g;
			for (k=i;k<=m;k++) a[k][j] += f*a[k][i];
		}
		for (j=i;j<=m;j++) a[j][i] *= g;
	} else for (j=i;j<=m;j++) a[j][i]=0.0;
	++a[i][i];
	}
	for (k=n;k>=1;k--) {	// Diagonalization of the bidiagonal form: Loop over
							// singular values, and over allowed iterations.
		for (its=1;its<=30;its++) {
			flag=1;
			for (l=k;l>=1;l--) {	// Test for splitting.
				nm=l-1;				// Note that rv1[1] is always zero.
				if ((double)(fabs(rv1[l])+anorm) == anorm) {
					flag=0;
					break;
				}
				if ((double)(fabs(w[nm])+anorm) == anorm) break;
			}
			if (flag) {
				c=0.0;		// Cancellation of rv1[l], if l > 1.
					s=1.0;
				for (i=l;i<=k;i++) {
					f=s*rv1[i];
					rv1[i]=c*rv1[i];
					if ((double)(fabs(f)+anorm) == anorm) break;
					g=w[i];
					h=pythag(f,g);
					w[i]=h;
					h=1.0/h;
					c=g*h;
					s = -f*h;
					for (j=1;j<=m;j++) {
						y=a[j][nm];
						z=a[j][i];
						a[j][nm]=y*c+z*s;
						a[j][i]=z*c-y*s;
					}
				}
			}
			z=w[k];
			if (l == k) {		// Convergence.
				if (z < 0.0) {  // Singular value is made nonnegative.
					w[k] = -z;
				for (j=1;j<=n;j++) v[j][k] = -v[j][k];
				}
				break;
			}
			if (its == 30) {free_vector(rv1,1,n); return False;}
			x=w[l];  // Shift from bottom 2-by-2 minor.
				nm=k-1;
			y=w[nm];
			g=rv1[nm];
			h=rv1[k];
			f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y);
			g=pythag(f,1.0);
			f=((x-z)*(x+z)+h*((y/(f+SIGN(g,f)))-h))/x;
			c=s=1.0;  // Next QR transformation:
			for (j=l;j<=nm;j++) {
				i=j+1;
				g=rv1[i];
				y=w[i];
				h=s*g;
				g=c*g;
				z=pythag(f,h);
				rv1[j]=z;
				c=f/z;
				s=h/z;
				f=x*c+g*s;
				g = g*c-x*s;
				h=y*s;
				y *= c;
				for (jj=1;jj<=n;jj++) {
					x=v[jj][j];
					z=v[jj][i];
					v[jj][j]=x*c+z*s;
					v[jj][i]=z*c-x*s;
				}
				z=pythag(f,h);
				w[j]=z;	 // Rotation can be arbitrary if z = 0.
					if (z) {
						z=1.0/z;
						c=f*z;
						s=h*z;
					}
					f=c*g+s*y;
					x=c*y-s*g;
					for (jj=1;jj<=m;jj++) {
						y=a[jj][j];
						z=a[jj][i];
						a[jj][j]=y*c+z*s;
						a[jj][i]=z*c-y*s;
					}
			}
			rv1[l]=0.0;
			rv1[k]=f;
			w[k]=x;
		}
	}
	free_vector(rv1,1,n);
	return True;
}

/******************************* auxiliary function *******************************/

double pythag(double a, double b)
// Computes (a2 + b2)1/2 without destructive underflow or overflow.
{
	double absa,absb;
	absa=fabs(a);
	absb=fabs(b);
	if (absa > absb) return absa*sqrt(1.0+SQR(absb/absa));
	else return (absb == 0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}

/*******************************CostFunction******************************************/

void calculate(double *x, int nvar, double *CostFunction, int nfun)
{
	int i;
	double *data, *solution;
	data = dvector(1,nfun);
	solution = dvector(1, nfun);
	data[1]=(-1.5)*(-1.5)+(-0.5)*(-0.5);
	data[2]=(-1.5)*(-1.5)+4.28*4.28;
	data[3]=0.34*0.34+(-0.5)*(-0.5);
	data[4]=0.34*0.34+4.28*4.28;
	solution[1] = x[1]*(-1.5)*(-1.5)+x[2]*(-0.5)*(-0.5);
	solution[2] = x[1]*(-1.5)*(-1.5)+x[2]*4.28*4.28;
	solution[3] = x[1]*0.34*0.34+x[2]*(-0.5)*(-0.5);
	solution[4] = x[1]*0.34*0.34+x[2]*4.28*4.28;
	for (i=1;i<=nfun;i++) {
		CostFunction[i] = solution[i] - data[i];
	}

}

