/*
 * output.c: Write simulation output data for to files.
 *
 * Author(s): Javier Nicolas Sanchez
 *            Mei Marker
 */

#include "output.h"
#include "model.h"

float result[NUM_VARS];

/*This indicates whether or not a process is active at each time. It is computed post-factor, 
  meaning after evalVars are computed, it re-computes the test conditions for each process to see if they are true or not.*/
int evaluated_proc[NUM_PROCS];

/**
 * Print result header, first all the variable names, then all the process names
 */
void
print_full_result_header (FILE * outStream)
{
  int i;
  fprintf (outStream, "t");
  for (i = 0; i < NUM_VARS; i++)
    fprintf (outStream, " %s", numeric_field_names[i]);
  for (i = 0; i < NUM_PROCS - 1; i++)
    fprintf (outStream, " %s", proc_names[i]);
  fprintf (outStream, "\n");
}

/** 
 * Print result data, first all the variable values, then all the process values (1/0 indicting active/inactive)
 */
void
print_full_output (FILE * outStream)
{
  int i;
  fprintf (outStream, "%f", t);
  for (i = 0; i < NUM_VARS; i++)
    fprintf (outStream, " %f", result[i]);
  for (i = 0; i < NUM_PROCS - 1; i++)
    fprintf (outStream, " %d", evaluated_proc[i]);
  fprintf (outStream, "\n");
}

/**
 * Print result header for all the variable names only
 */
void
print_vars_result_header (FILE * outStream)
{
  int i;
  fprintf (outStream, "t");
  for (i = 0; i < NUM_VARS; i++)
    fprintf (outStream, " %s", numeric_field_names[i]);
  fprintf (outStream, "\n");
}

/** 
 * print result data for all the variable names only
 */
void
print_vars_output (FILE * outStream)
{
  int i;
  fprintf (outStream, "%f", t);
  for (i = 0; i < NUM_VARS; i++)
    fprintf (outStream, " %f", result[i]);
  fprintf (outStream, "\n");
}
