#!/usr/bin/perl -w

#-------------------------------------------------------------------------------#
# runGenerateModels.pl : a wrapper script for revision engine                   #
# Author: Tamar T. Shinar, 1/2004                                               #
#-------------------------------------------------------------------------------#

#-------------------------------------------------------------------------------#
#                        declarations                                           #
#-------------------------------------------------------------------------------#
sub usage();
sub getCheckArgs();
sub config();
sub preRunCmd();
sub runCmd();
sub postRunCmd($);
use Cwd;

my ($revisionEnginePath, $revisionEngine, $matlabLibPath);
my ($processLibrary, $revisionModels, $typeLibrary, $data, $outFileName, $dataOutFileName);
my ($tmpModelOutFile, $tmpDataOutFile);

#-------------------------------------------------------------------------------#
#                       main                                                    #
#-------------------------------------------------------------------------------#
$oldld = $ENV{'LD_LIBRARY_PATH'}; 
config();
print $ENV{'LD_LIBRARY_PATH'};
getCheckArgs();
preRunCmd();
my $result = runCmd();

$ENV{'LD_LIBRARY_PATH'}=$oldld;

# exit value : 0 for success
my $exitValue = postRunCmd($result);
print "$0 : exitValue is : $exitValue\n";
exit($exitValue);

#-------------------------------------------------------------------------------#
#                                subroutines                                    #
#-------------------------------------------------------------------------------#

# set some configuration parameters
sub config()
{
  # set default parameters
  $revisionEnginePath = "./RPM";
  $revisionEngine = "generateModels";
  $dir = cwd();
  $matlabLibPath = $dir . "/RPM/bin/glnx86";
  
  # set the LD_LIBRARY_PATH environment variable
  $ENV{LD_LIBRARY_PATH} = $matlabLibPath;
}

sub usage()
{
  die <<USAGE
  Usage: $0 processLibrary revisionModels typeLibrary data modelOutFileName dataOutFileName
USAGE
  ;
}

# check for existence of all input files
sub getCheckArgs()
{
  # check for correct number of arguments
  usage() unless (@ARGV == 6);

  # get command line arguments
  $processLibrary = $ARGV[0];
  $revisionModels = $ARGV[1];
  $typeLibrary = $ARGV[2];
  $data = $ARGV[3];
  # names of output files that the caller expects
  $outFileName = $ARGV[4];
  $dataOutFileName = $ARGV[5];

  # check that the input files exist
  my $checkFile;
  if (! -f ($checkFile = $processLibrary) or
      ! -f ($checkFile = $revisionModels) or
      ! -f ($checkFile = $typeLibrary) or 
      ! -f ($checkFile = $data) ) 
  {
    die "$checkFile: $!\n";
  } 
}

sub preRunCmd()
{
  # name of output files that revision engine will create
  $tmpModelOutFile = "rev.out";
  $tmpDataOutFile = "init." . $tmpModelOutFile;

  # removed the temporary output files if they exist
  unlink($tmpModelOutFile) if (-f $tmpModelOutFile);
  unlink($tmpDataOutFile) if (-f $tmpDataOutFile);
}

sub runCmd()
{
  # run the command
  my $command = "cd $revisionEnginePath; ./$revisionEngine $processLibrary $revisionModels $typeLibrary $data $tmpModelOutFile";
  #print "Executing:\n\t$command\n";

  my $result = system($command);

  return $result;
}

sub postRunCmd($)
{
  my ($cmdResult) = @_;

  # result of 0 indicates success
  my $result = 0;

  #Note that die messaages apparently don't appear in the console since 
  #this script is called by the GUI.  Therefore we print error to 
  #SDTOUT before calling die.
  if ($cmdResult) {
      print "$0: Revision engine reported failure on exit.  Exit value $cmdResult";
      $! = 7; #set exit code for die.  Could use this to provide
              #more descriptive error message in the GUI.
      die "Revision engine failed.\n";
  } elsif (! -f "$revisionEnginePath/$tmpModelOutFile") {
      print "$0: Revision engine did not produce a valid model output file.  ";
      print "Expected $revisionEnginePath/$tmpModelOutFile to be a regular file, but it is not or does not exist.\n";
      $! = 8; #set exit code for die.  Could use this to provide
              #more descriptive error messages in the GUI.
      die "Revision engine did not produce valid model output file.\n";
  } elsif (! -f "$revisionEnginePath/$tmpDataOutFile") {
      print "$0: Revision engine did not produce a valid data output file.  Expected $revisionEnginePath/$tmpDataOutFile to be a regular file, but it is irregular or does not exist.";
      $! = 9;  #set exit code for die.
      die "Revision engine did not produce a valid data output file.\n";
  } else {
      #Revision engine reported success, and the expected output files exist.
      print "$0: revision engine ran successfully\n";
    
      # copy the output files to the location the user expects
      # terminate if copying fails
      $renameOutfileSucceeded = rename("$revisionEnginePath/$tmpModelOutFile", $outFileName);
      if (!$renameOutfileSucceeded) {
	  print "$0: Error - could not copy output to $outFileName: $!\n";
	  $! = 10; #assign exit value for die
	  die "Error - could not copy output to $outFileName: $!\n";
      }

      $renameDataOutfileSucceeded = rename("$revisionEnginePath/$tmpDataOutFile", $dataOutFileName);
      if (!$renameDataOutfileSucceeded) {
	  print "$0: Error - could not copy output to $dataOutFileName: $!\n";
	  $! = 11; #assign exit value for die.
	  die "Error - could not copy output to $dataOutFileName: $!\n";
      }
  }

  # check if the expected output files are there
  if (! -f $outFileName or ! -f $dataOutFileName)
  {
    $result = 1;
  }

  return $result;
}


